@@ -5,6 +5,7 @@
 from __future__ import absolute_import, division, print_function
 from muninn._compat import string_types as basestring
 
+import collections
 import copy
 import datetime
 import errno
@@ -57,6 +58,7 @@ class _ArchiveConfig(Mapping):
     namespace_extensions = _ExtensionList(optional=True)
     product_type_extensions = _ExtensionList(optional=True)
     remote_backend_extensions = _ExtensionList(optional=True)
+    hook_extensions = _ExtensionList(optional=True)
     auth_file = Text(optional=True)
 
 
@@ -112,6 +114,7 @@ def create(configuration):
     namespace_extensions = options.pop("namespace_extensions", [])
     product_type_extensions = options.pop("product_type_extensions", [])
     remote_backend_extensions = options.pop("remote_backend_extensions", [])
+    hook_extensions = options.pop("hook_extensions", [])
     archive = Archive(backend=backend, storage=storage, **options)
 
     # Register core namespace.
@@ -144,6 +147,15 @@ def create(configuration):
         except AttributeError:
             raise Error("extension %r does not implement the remote backend extension API" % name)
 
+    # Register hook extensions.
+    for name in hook_extensions:
+        extension = _load_extension(name)
+        try:
+            for hook_extension in extension.hook_extensions():
+                archive.register_hook_extension(hook_extension, extension.hook_extension(hook_extension))
+        except AttributeError:
+            raise Error("extension %r does not implement the hook extension API" % name)
+
     return archive
 
 
@@ -158,6 +170,7 @@ class Archive(object):
         self._namespace_schemas = {}
         self._product_type_plugins = {}
         self._remote_backend_plugins = copy.copy(remote.REMOTE_BACKENDS)
+        self._hook_extensions = collections.OrderedDict()
         self._export_formats = set()
 
         self._database = backend
@@ -165,11 +178,124 @@ class Archive(object):
 
         self._storage = storage
 
-    def __enter__(self):
-        return self
+    def register_namespace(self, namespace, schema):
+        """Register a namespace.
 
-    def __exit__(self, type, value, traceback):
-        self.close()
+        Arguments:
+        namespace -- Name of the namespace.
+        schema    -- Schema definition of the namespace.
+
+        """
+        if not re.match(r"[a-z][_a-z]*(\.[a-z][_a-z]*)*", namespace):
+            raise ValueError("invalid namespace name %s" % namespace)
+        if namespace in self._namespace_schemas:
+            raise Error("redefinition of namespace: \"%s\"" % namespace)
+
+        self._namespace_schemas[namespace] = schema
+
+    def namespace_schema(self, namespace):
+        """Return the schema definition of a namespace."""
+        try:
+            return self._namespace_schemas[namespace]
+        except KeyError:
+            raise Error("undefined namespace: \"%s\"; defined namespaces: %s" %
+                        (namespace, util.quoted_list(self._namespace_schemas.keys())))
+
+    def namespaces(self):
+        """Return a list of supported namespaces."""
+        return list(self._namespace_schemas.keys())
+
+    def register_product_type(self, product_type, plugin):
+        """Register a product type.
+
+        Arguments:
+        product_type -- Product type.
+        plugin       -- Reference to an object that implements the product type plugin API and as such takes care of
+                        the details of extracting product properties from products of the specified product type.
+
+        """
+        if product_type in self._product_type_plugins:
+            raise Error("redefinition of product type: \"%s\"" % product_type)
+
+        # Quick verify of the plugin interface
+        for attr in ['use_enclosing_directory']:
+            if not hasattr(plugin, attr):
+                raise Error("missing '%s' attribute in plugin for product type \"%s\"" % (attr, product_type))
+
+        if hasattr(plugin, 'use_hash') and not hasattr(plugin, 'hash_type'):
+            warnings.warn("'use_hash' option is deprecated (use 'hash_type')")
+
+        methods = ['identify', 'analyze', 'archive_path']
+        if plugin.use_enclosing_directory:
+            methods += ['enclosing_directory']
+        for method in methods:
+            if not hasattr(plugin, method):
+                raise Error("missing '%s' method in plugin for product type \"%s\"" % (method, product_type))
+
+        self._product_type_plugins[product_type] = plugin
+        self._update_export_formats(plugin)
+
+    def product_type_plugin(self, product_type):
+        """Return a reference to the product type plugin for a product type."""
+        try:
+            return self._product_type_plugins[product_type]
+        except KeyError:
+            raise Error("undefined product type: \"%s\"; defined product types: %s" %
+                        (product_type, util.quoted_list(self._product_type_plugins.keys())))
+
+    def product_types(self):
+        """Return a list of supported product types."""
+        return list(self._product_type_plugins.keys())
+
+    def register_remote_backend(self, remote_backend, plugin):
+        """Register a remote backend.
+
+        Arguments:
+        remote_backend -- Remote backend.
+        plugin         -- Reference to an object that implements the remote backend plugin API and as such takes care of
+                          the details of extracting product properties from products of the specified remote backend.
+
+        """
+        if remote_backend in self._remote_backend_plugins:
+            raise Error("redefinition of remote backend: \"%s\"" % remote_backend)
+
+        self._remote_backend_plugins[remote_backend] = plugin
+
+    def remote_backend(self, remote_backend):
+        """Return the schema definition of a remote backend."""
+        try:
+            return self._remote_backend_plugins[remote_backend]
+        except KeyError:
+            raise Error("undefined remote backend: \"%s\"; defined remote backends: %s" %
+                        (remote_backend, util.quoted_list(self._remote_backend.keys())))
+
+    def remote_backends(self):
+        """Return a list of supported remote backends."""
+        return list(self._remote_backend_plugins.keys())
+
+    def register_hook_extension(self, hook_extension, plugin):
+        """Register a hook extension.
+
+        Arguments:
+        hook_extension -- Hook extension name
+        plugin         -- Reference to an object that implements the hook extension plugin API
+        """
+        if hook_extension in self._hook_extensions:
+            raise Error("redefinition of hook extension: \"%s\"" % hook_extension)
+
+        self._hook_extensions[hook_extension] = plugin
+
+    def hook_extension(self, hook_extension):
+        """Return the hook extension with the specified name."""
+        try:
+            return self._hook_extensions[hook_extension]
+        except KeyError:
+            raise Error("undefined hook extension: \"%s\"; defined hook extensions: %s" %
+                        (hook_extension, util.quoted_list(self._hook_extensions.keys())))
+
+    def hook_extensions(self):
+        """Return a list of supported hook extensions."""
+        return list(self._hook_extensions.keys())
 
     def _plugin_hash_type(self, plugin):
         if hasattr(plugin, 'hash_type'):
@@ -266,10 +392,9 @@ class Archive(object):
         # Remove any data in storage associated with the product.
         self._remove(product)
 
-        # Run the post remove hook (if defined by the product type plug-in).
+        # Run the post remove hook (if defined by the product type plug-in or hook extensions).
         plugin = self.product_type_plugin(product.core.product_type)
-        if hasattr(plugin, "post_remove_hook"):
-            plugin.post_remove_hook(self, product)
+        self._run_hooks('post_remove_hook', plugin, product, reverse=True)
 
     def _relocate(self, product, properties=None):
         """Relocate a product to the archive_path reported by the product type plugin.
@@ -293,7 +418,7 @@ class Archive(object):
             # If the product does not exist, do not consider this an error.
             return
 
-        # Remove the data associated with the product from disk.
+        # Remove the data associated with the product from storage.
         self._storage.delete(product_path, product)
 
     def _retrieve(self, product, target_path, use_symlinks=False):
@@ -364,7 +489,7 @@ class Archive(object):
         """
         return self._database.count(where, parameters)
 
-    def create_properties(self, properties):
+    def create_properties(self, properties, disable_hooks=False):
         """ Create record for product in the product catalogue.
             An important side effect of this operation is that it
             will fail if:
@@ -376,6 +501,12 @@ class Archive(object):
         self._update_metadata_date(properties)
         self._database.insert_product_properties(properties)
 
+        # Run the post create hook (if defined by the product type plug-in or hook extensions).
+        if not disable_hooks:
+            plugin = self.product_type_plugin(properties.core.product_type)
+            self._run_hooks('post_create_hook', plugin, properties)
+
+
     def delete_properties(self, where="", parameters={}):
         """Remove properties for one or more products from the catalogue. Return the number of products removed.
 
@@ -638,11 +769,26 @@ class Archive(object):
 
         # Remove existing product with the same product type and name before ingesting
         if force:
-            self.remove(where="core.product_type == @product_type and core.product_name == @product_name",
-                        parameters={'product_type': properties.core.product_type,
-                                    'product_name': properties.core.product_name}, force=True)
+            existing = self.search(where="core.product_type == @product_type and core.product_name == @product_name",
+                                   parameters={'product_type': properties.core.product_type,
+                                               'product_name': properties.core.product_name})
+            if existing:
+                existing = existing[0]
+                ingest_path = os.path.dirname(paths[0])
+                if len(paths) > 1:
+                    ingest_path = os.path.dirname(ingest_path)
+                current_path = self.root()
+                if existing.core.archive_path:
+                    current_path = os.path.join(current_path, existing.core.archive_path)
+                if existing.core.archive_path != properties.core.archive_path:
+                    raise Error('cannot force ingest because of archive_path mismatch')
+                if ingest_path == current_path:
+                    # do not remove the product being ingested (only remove from catalogue)
+                    self.delete_properties_by_uuid(existing.core.uuid)
+                else:
+                    self.remove_by_uuid(existing.core.uuid, force=True)
 
-        self.create_properties(properties)
+        self.create_properties(properties, disable_hooks=True)
 
         # Try to determine the product hash and ingest the product into the archive.
         try:
@@ -683,15 +829,31 @@ class Archive(object):
         # Set product tags.
         self._database.tag(properties.core.uuid, tags)
 
-        # Run the post ingest hook (if defined by the product type plug-in).
-        #
-        # Note that hasattr() is used instead of a try + except block that swallows AttributeError to avoid hiding
-        # AttributeError instances raised by the plug-in.
-        if hasattr(plugin, "post_ingest_hook"):
-            plugin.post_ingest_hook(self, properties)
+        # Run post create/ingest hooks (if defined by the product type plug-in or hook extensions).
+        if not ingest_product:
+            self._run_hooks('post_create_hook', plugin, properties)
+        else:
+            self._run_hooks('post_ingest_hook', plugin, properties)
 
         return properties
 
+    def _hook_exists(self, hook_name, plugin):
+        plugins = [plugin] + list(self._hook_extensions.values())
+        for plugin in plugins:
+            if hasattr(plugin, hook_name):
+                return True
+        return False
+
+    def _run_hooks(self, hook_name, plugin, properties, reverse=False):
+        plugins = [plugin] + list(self._hook_extensions.values())
+        if reverse:
+            plugins = reversed(plugins)
+
+        for plugin in plugins:
+            hook_method = getattr(plugin, hook_name, None)
+            if hook_method is not None:
+                hook_method(self, properties)
+
     def link(self, uuid_, source_uuids):
         """Link a product to one or more source products."""
         if isinstance(source_uuids, uuid.UUID):
@@ -699,18 +861,6 @@ class Archive(object):
 
         self._database.link(uuid_, source_uuids)
 
-    def namespace_schema(self, namespace):
-        """Return the schema definition of a namespace."""
-        try:
-            return self._namespace_schemas[namespace]
-        except KeyError:
-            raise Error("undefined namespace: \"%s\"; defined namespaces: %s" %
-                        (namespace, util.quoted_list(self._namespace_schemas.keys())))
-
-    def namespaces(self):
-        """Return a list of supported namespaces."""
-        return list(self._namespace_schemas.keys())
-
     def prepare(self, force=False):
         """Prepare the archive for (first) use.
 
@@ -765,18 +915,6 @@ class Archive(object):
 
         return os.path.join(self._storage.global_prefix, self._product_path(product))
 
-    def product_type_plugin(self, product_type):
-        """Return a reference to the product type plugin for a product type."""
-        try:
-            return self._product_type_plugins[product_type]
-        except KeyError:
-            raise Error("undefined product type: \"%s\"; defined product types: %s" %
-                        (product_type, util.quoted_list(self._product_type_plugins.keys())))
-
-    def product_types(self):
-        """Return a list of supported product types."""
-        return list(self._product_type_plugins.keys())
-
     def pull(self, where="", parameters={}, verify_hash=False):
         """Pull one or more remote products into the archive.
         Return the number of products pulled.
@@ -831,9 +969,8 @@ class Archive(object):
                     raise Error("pulled product '%s' (%s) has incorrect hash" %
                                 (product.core.product_name, product.core.uuid))
 
-            # Run the post pull hook (if defined by the product type plug-in).
-            if hasattr(plugin, "post_pull_hook"):
-                plugin.post_pull_hook(self, product)
+            # Run the post pull hook (if defined by the product type plug-in or hook extensions).
+            self._run_hooks('post_pull_hook', plugin, product)
 
         return len(queue)
 
@@ -914,15 +1051,12 @@ class Archive(object):
         # Update tags.
         self.tag(product.core.uuid, tags)
 
-        # Run the post ingest hook (if defined by the product type plug-in).
-        #
-        # Note that hasattr() is used instead of a try + except block that swallows AttributeError to avoid hiding
-        # AttributeError instances raised by the plug-in.
-        if not disable_hooks and hasattr(plugin, "post_ingest_hook"):
+        # Run the post ingest hook (if defined by the product type plug-in or hook extensions).
+        if not disable_hooks and self._hook_exists('post_ingest_hook', plugin):
             product.update(properties)
             if 'hash' not in product.core:
                 product.core.hash = None
-            plugin.post_ingest_hook(self, product)
+            self._run_hooks('post_ingest_hook', plugin, product)
 
     def rebuild_pull_properties(self, uuid, verify_hash=False, disable_hooks=False, use_current_path=False):
         """Refresh products by re-running the pull, but using the existing products stored in the archive.
@@ -961,84 +1095,13 @@ class Archive(object):
                 raise Error("pulled product '%s' (%s) has incorrect hash" %
                             (product.core.product_name, product.core.uuid))
 
-        # Run the post pull hook (if defined by the product type plug-in).
-        if not disable_hooks and hasattr(plugin, "post_pull_hook"):
-            plugin.post_pull_hook(self, product)
-
-    def register_namespace(self, namespace, schema):
-        """Register a namespace.
-
-        Arguments:
-        namespace -- Name of the namespace.
-        schema    -- Schema definition of the namespace.
-
-        """
-        if not re.match(r"[a-z][_a-z]*(\.[a-z][_a-z]*)*", namespace):
-            raise ValueError("invalid namespace name %s" % namespace)
-        if namespace in self._namespace_schemas:
-            raise Error("redefinition of namespace: \"%s\"" % namespace)
-
-        self._namespace_schemas[namespace] = schema
-
-    def register_product_type(self, product_type, plugin):
-        """Register a product type.
-
-        Arguments:
-        product_type -- Product type.
-        plugin       -- Reference to an object that implements the product type plugin API and as such takes care of
-                        the details of extracting product properties from products of the specified product type.
-
-        """
-        if product_type in self._product_type_plugins:
-            raise Error("redefinition of product type: \"%s\"" % product_type)
-
-        # Quick verify of the plugin interface
-        for attr in ['use_enclosing_directory']:
-            if not hasattr(plugin, attr):
-                raise Error("missing '%s' attribute in plugin for product type \"%s\"" % (attr, product_type))
-
-        if hasattr(plugin, 'use_hash') and not hasattr(plugin, 'hash_type'):
-            warnings.warn("'use_hash' option is deprecated (use 'hash_type')")
-
-        methods = ['identify', 'analyze', 'archive_path']
-        if plugin.use_enclosing_directory:
-            methods += ['enclosing_directory']
-        for method in methods:
-            if not hasattr(plugin, method):
-                raise Error("missing '%s' method in plugin for product type \"%s\"" % (method, product_type))
-
-        self._product_type_plugins[product_type] = plugin
-        self._update_export_formats(plugin)
-
-    def register_remote_backend(self, remote_backend, plugin):
-        """Register a remote backend.
-
-        Arguments:
-        remote_backend -- Remote backend.
-        plugin         -- Reference to an object that implements the remote backend plugin API and as such takes care of
-                          the details of extracting product properties from products of the specified remote backend.
-
-        """
-        if remote_backend in self._remote_backend_plugins:
-            raise Error("redefinition of remote backend: \"%s\"" % remote_backend)
-
-        self._remote_backend_plugins[remote_backend] = plugin
-
-    def remote_backend(self, remote_backend):
-        """Return the schema definition of a remote_backend."""
-        try:
-            return self._remote_backend_plugins[remote_backend]
-        except KeyError:
-            raise Error("undefined remote backend: \"%s\"; defined remote backends: %s" %
-                        (remote_backend, util.quoted_list(self._remote_backend.keys())))
-
-    def remote_backends(self):
-        """Return a list of supported remote_backends."""
-        return list(self._remote_backend_plugins.keys())
+        # Run the post pull hook (if defined by the product type plug-in or hook extensions).
+        if not disable_hooks:
+            self._run_hooks('post_pull_hook', plugin, product)
 
     def remove(self, where="", parameters={}, force=False):
-        """Remove one or more products from the archive, both from disk as well as from the product catalogue. Return
-        the number of products removed.
+        """Remove one or more products from the archive, both from storage as well as from the product catalogue.
+        Return the number of products removed.
 
         NB. Depending on product type specific cascade rules, removing a product can result in one or more derived
         products being removed (or stripped) along with it. Such products are _not_ included in the returned count.
@@ -1205,7 +1268,7 @@ class Archive(object):
         return self._database.source_products(uuid)
 
     def strip(self, where="", parameters={}, force=False):
-        """Remove one or more products from disk only (not from the product catalogue). Return the number of products
+        """Remove one or more products from storage only (not from the product catalogue). Return the number of products
         stripped.
 
         NB. Depending on product type specific cascade rules, stripping a product can result in one or more derived
@@ -1236,7 +1299,7 @@ class Archive(object):
         return len(products)
 
     def strip_by_name(self, product_name):
-        """Remove one or more products from disk only (not from the product catalogue).
+        """Remove one or more products from storage only (not from the product catalogue).
 
         This is a convenience function that is equivalent to:
 
@@ -1254,7 +1317,7 @@ class Archive(object):
         return count
 
     def strip_by_uuid(self, uuid):
-        """Remove a product from disk only (not from the product catalogue).
+        """Remove a product from storage only (not from the product catalogue).
 
         This is a convenience function that is equivalent to:
 
@@ -1384,3 +1447,9 @@ class Archive(object):
                     failed_products.append(product.core.uuid)
 
         return failed_products
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.close()
