@@ -54,20 +54,33 @@ def _get_db_type_id(connection, typename):
     return type_id
 
 
-def geometry_recv(data, offset, length):
+def geometry_recv(data, offset, length): # TODO binary send/recv needed for pg8000 <= 1.15
     return ewkb.decode_ewkb(data[offset:offset+length])
 
 
+def geometry_recv_hex(data):
+    return ewkb.decode_hexewkb(data)
+
+
 def geometry_send(geometry):
     return ewkb.encode_ewkb(geometry)
 
 
+def geometry_send_hex(geometry):
+    return ewkb.encode_hexewkb(geometry)
+
+
 def _connect_pg8000(connection_string):
     kwargs = dict(arg.split('=') for arg in connection_string.split(' '))
     _connection = pg8000.connect(**kwargs)
 
     geography_oid = _get_db_type_id(_connection, "geography")
-    _connection.pg_types[geography_oid] = (pg8000.core.FC_BINARY, geometry_recv)
+
+    if hasattr(_connection, 'register_in_adapter'):
+        _connection.register_in_adapter(geography_oid, geometry_recv_hex)
+    else:
+        _connection.pg_types[geography_oid] = (pg8000.core.FC_BINARY, geometry_recv)
+
     for type_ in (
         geometry.Point,
         geometry.Polygon,
@@ -76,7 +89,10 @@ def _connect_pg8000(connection_string):
         geometry.MultiPolygon,
         geometry.MultiLineString,
     ):
-        _connection.py_types[type_] = (geography_oid, pg8000.core.FC_BINARY, geometry_send)
+        if hasattr(_connection, 'register_out_adapter'):
+            _connection.register_out_adapter(type_, geography_oid, geometry_send_hex)
+        else:
+            _connection.py_types[type_] = (geography_oid, pg8000.core.FC_BINARY, geometry_send)
 
     return _connection
 
@@ -526,8 +542,18 @@ class PostgresqlBackend(object):
             return "TO_CHAR(%s, 'MM')" % column_name
         if subscript == 'yearmonth':
             return "TO_CHAR(%s, 'YYYY-MM')" % column_name
+        if subscript == 'day':
+            return "TO_CHAR(%s, 'DD')" % column_name
         if subscript == 'date':
             return "TO_CHAR(%s, 'YYYY-MM-DD')" % column_name
+        if subscript == 'hour':
+            return "TO_CHAR(%s, 'HH24')" % column_name
+        if subscript == 'minute':
+            return "TO_CHAR(%s, 'MI')" % column_name
+        if subscript == 'second':
+            return "TO_CHAR(%s, 'SS')" % column_name
+        if subscript == 'time':
+            return "TO_CHAR(%s, 'HH24:MI:SS')" % column_name
         # text
         if subscript == 'length':
             return "CHAR_LENGTH(%s)" % column_name
